@@ -2,7 +2,7 @@
 All of the documentation and software included in the Mserv releases is
 copyrighted by James Ponder <james@squish.net>.
 
-Copyright 1999, 2000 James Ponder.  All rights reserved.
+Copyright 1999-2003 James Ponder.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -1633,7 +1633,7 @@ static void mserv_cmd_volume(t_client *cl, const char *ru, const char *line)
   (void)ru;
   if (*line && cl->userlevel == level_guest) {
     mserv_response(cl, "ACLFAIL", NULL);
-  } else if ((val = mserv_setmixer(cl, SOUND_MIXER_VOLUME, line)) != -1) {
+  } else if ((val = mserv_setmixer(cl, SOUND_MIXER_PCM, line)) != -1) {
     if (!*line) {
       mserv_response(cl, "VOLCUR", "%d", val & 0xFF);
     } else {
@@ -2789,6 +2789,8 @@ static void mserv_cmd_reset(t_client *cl, const char *ru, const char *line)
     mserv_response(cl, "RESETER", NULL);
     return;
   }
+  mserv_broadcast("RESETGO", "%s", cl->user);
+  mserv_flush();
   mserv_log("Reset initiated by %s", cl->user);
   mserv_reset();
   mserv_broadcast("RESET", "%s", cl->user);
