@@ -1,6 +1,6 @@
 
-#define MP3ID3_TITLELEN 30
-#define MP3ID3_ARTISTLEN 30
+#define MP3ID3_TITLELEN NAMELEN
+#define MP3ID3_ARTISTLEN AUTHORLEN
 #define MP3ID3_ALBUMLEN 30
 #define MP3ID3_YEARLEN 4
 #define MP3ID3_COMMENTLEN 30
