@@ -2,7 +2,7 @@
 All of the documentation and software included in the Mserv releases is
 copyrighted by James Ponder <james@squish.net>.
 
-Copyright 1999, 2000 James Ponder.  All rights reserved.
+Copyright 1999-2003 James Ponder.  All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
@@ -60,6 +60,7 @@ met:
 #include <dirent.h>
 #include <math.h>
 #include <sys/ioctl.h>
+#include <time.h>
 
 #include "mserv.h"
 #include "misc.h"
@@ -150,6 +151,7 @@ static int mserv_createdir(const char *path);
 static void mserv_sendwrap(t_client *cl, const char *string, int wrapwidth,
 			   const char *indentstr, int nextwidth);
 static void mserv_strtoprintable(char *string);
+static const char *mserv_getplayer(char *fname);
 #ifdef SOUNDCARD
 static int mserv_setvolume(int vol);
 static int mserv_readvolume(void);
@@ -275,7 +277,7 @@ int main(int argc, char *argv[])
   }    
 
   if (mserv_verbose)
-    printf("Mserv version " VERSION " (c) James Ponder 1999-2000\n");
+    printf("Mserv version " VERSION " (c) James Ponder 1999-2003\n");
 
   if (!mserv_root) {
     if ((ps = getpwuid(getuid())) == NULL || ps->pw_dir[0] == '\0') {
@@ -603,7 +605,7 @@ static void mserv_mainloop(void)
     cl->authed = 0;
     cl->mode = mode_computer;
     cl->user[0] = '\0';
-    mserv_send(cl, "200 Mserv " VERSION " (c) James Ponder 2000 - "
+    mserv_send(cl, "200 Mserv " VERSION " (c) James Ponder 1999-2003 - "
 	       "Type: USER <username>\r\n", 0);
     mserv_send(cl, ".\r\n", 0);
   }
@@ -1633,7 +1635,7 @@ static void mserv_scandir_recurse(const char *pathname)
   tnum = 0;
   toomany = 0;
 
-  /* find .mp3 files and use .trk files if they exist */
+  /* find music files and use .trk files if they exist */
 
   if ((unsigned int)snprintf(fullpath, 1024, "%s/%s",
 			     opt_path_tracks, pathname) >= 1024) {
@@ -1647,13 +1649,16 @@ static void mserv_scandir_recurse(const char *pathname)
   }
   while((ent = readdir(dir))) {
     llen = strlen(ent->d_name);
+    /* ignore dot files */
     if (ent->d_name[0] == '.')
       continue;
+    /* construct full path */
     if ((unsigned int)snprintf(fullpath, 1024, "%s/%s%s", opt_path_tracks,
 			       pathname, ent->d_name) >= 1024) {
       mserv_log("fullpath buffer too small");
       return;
     }
+    /* get details about this file */
     if (stat(fullpath, &buf)) {
       perror("stat");
       mserv_log("Unable to stat '%s'", fullpath);
@@ -1661,33 +1666,38 @@ static void mserv_scandir_recurse(const char *pathname)
     }
     filename = fullpath+strlen(opt_path_tracks)+1;
     if (S_ISDIR(buf.st_mode)) {
+      /* recurse into directories */
       strcat(filename, "/");
       mserv_scandir_recurse(filename);
-    } else if (!toomany && S_ISREG(buf.st_mode)) {
-      if (llen > 4 && !stricmp(".mp3", ent->d_name+llen-4)) {
-	if (tnum >= TRACKSPERALBUM) {
-	  mserv_log("The limit of tracks per album was reached, and some "
-		    "tracks were discarded. To increase this limit "
-		    "recompile with TRACKSPERALBUM set higher.  A better "
-		    "solution is to sub-divide your tracks into more "
-		    "directories.");
-	  toomany = 1;
-	  break;
-	}
-	if (mserv_verbose)
-	  mserv_log("Track file: %s", fullpath);
-	if ((tracks[tnum] = mserv_loadtrk(filename)) == NULL) {
-	  mserv_log("Unable to add track '%s'", fullpath);
-	} else {
-	  tracks[tnum]->id = mserv_nextid_track++;
-	  tracks[tnum]->n_album = mserv_nextid_album;
-	  tracks[tnum]->n_track = tnum+1;
-	  tracks[tnum]->next = mserv_tracks;
-	  mserv_tracks = tracks[tnum++];
-	}
-	flag = 1; /* there is at least one track in this directory */
-      }
+      continue;
     }
+    if (toomany || !S_ISREG(buf.st_mode))
+      /* ignore files once we're got too many or they aren't regular */
+      continue;
+    /* we have found a normal track - check there is a player for it */
+    if (mserv_getplayer(ent->d_name) == NULL)
+      continue; /* extension not found */
+    if (tnum >= TRACKSPERALBUM) {
+      mserv_log("The limit of tracks per album was reached, and some "
+		"tracks were discarded. To increase this limit "
+		"recompile with TRACKSPERALBUM set higher.  A better "
+		"solution is to sub-divide your tracks into more "
+		"directories.");
+      toomany = 1;
+      break;
+    }
+    if (mserv_verbose)
+      mserv_log("Track file: %s", fullpath);
+    if ((tracks[tnum] = mserv_loadtrk(filename)) == NULL) {
+      mserv_log("Unable to add track '%s'", fullpath);
+    } else {
+      tracks[tnum]->id = mserv_nextid_track++;
+      tracks[tnum]->n_album = mserv_nextid_album;
+      tracks[tnum]->n_track = tnum+1;
+      tracks[tnum]->next = mserv_tracks;
+      mserv_tracks = tracks[tnum++];
+    }
+    flag = 1; /* there is at least one track in this directory */
   }
   closedir(dir);
   /* load album, but only if there is an album file or flag is set */
@@ -2219,6 +2229,7 @@ int mserv_playnext(void)
   t_supinfo *history;
   char *str[21];
   char *strbuf;
+  const char *player;
 
   if (mserv_playing.track) {
     mserv_abortplay();
@@ -2311,11 +2322,12 @@ int mserv_playnext(void)
   }
   snprintf(fullpath, MAXFNAME, "%s/%s", opt_path_tracks,
 	   mserv_playing.track->filename);
-  if ((strbuf = malloc(strlen(opt_player)+1)) == NULL) {
+  player = mserv_getplayer(fullpath);
+  if ((strbuf = malloc(strlen(player)+1)) == NULL) {
     mserv_broadcast("MEMORY", NULL);
     return 1;
   }
-  strcpy(strbuf, opt_player);
+  strcpy(strbuf, player);
   if ((i = mserv_split(str, 21, strbuf, " ")) >= 21) {
     mserv_log("Internal error - too many parameters for player");
     exit(1);
@@ -2344,8 +2356,8 @@ int mserv_playnext(void)
     }
     execv(str[0], str);
     fprintf(stderr, "%s: Unable to execv '%s': '%s'", progname,
-	    opt_player, strerror(errno));
-    exit(0);
+	    player, strerror(errno));
+    exit(1);
   } else if (pid == -1) {
     perror("fork");
     mserv_log("Unable to fork: '%s'", strerror(errno));
@@ -2367,6 +2379,23 @@ int mserv_playnext(void)
   return 0;
 }
 
+/* get player string from filename or NULL for no player */
+
+static const char *mserv_getplayer(char *fname)
+{
+  char *dot;
+  char playername[32];
+  const char *player;
+
+  if ((dot = strrchr(fname, '.')) == NULL)
+    return NULL;
+  snprintf(playername, 32, "player_%s", dot+1);
+  if ((player = conf_getvalue(playername)) == NULL)
+    return NULL;
+  /* now convert player name (e.g. "mpg123") to full path and options */
+  return conf_getvalue(player); /* NULL or player invocation string */
+}
+
 void mserv_abortplay(void)
 {
   int pid, st;
@@ -2413,7 +2442,7 @@ void mserv_abortplay(void)
 	  if ((pid = waitpid(mserv_playingpid, &st,
 			     WNOHANG)) == -1) {
 	    mserv_log("waitpid failure (%d): %s", errno, strerror(errno));
-	  } else if (pid) {
+          } else if (pid) {
 	    goto STOPPED;
 	  }
 	}
@@ -3285,7 +3314,7 @@ static int mserv_readvolume(void)
 
   if (!(mixer_fd = open(opt_path_mixer, O_RDWR, 0)))
     return -1;
-  if (ioctl(mixer_fd, MIXER_READ(SOUND_MIXER_VOLUME), &vol) == -1) {
+  if (ioctl(mixer_fd, MIXER_READ(SOUND_MIXER_PCM), &vol) == -1) {
     close(mixer_fd); /* ignore errors */
     return -1;
   }
@@ -3300,7 +3329,7 @@ static int mserv_setvolume(int vol)
 
   if (!(mixer_fd = open(opt_path_mixer, O_RDWR, 0)))
     return -1;
-  if (ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_VOLUME), &newval) == -1) {
+  if (ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_PCM), &newval) == -1) {
     close(mixer_fd); /* ignore errors */
     return -1;
   }
@@ -3328,15 +3357,19 @@ int mserv_setmixer(t_client *cl, int what, const char *line)
     return -1;
   }
   curval = curval & 0xff;
-  if (!*line)
+  if (!*line) {
+    close(mixer_fd);
     return curval;
+  }
   if (*line == '+' || *line == '-') {
     type = *line == '+' ? 1 : -1;
     param = 1;
     p = line+1;
     if (*line == *p) {
-      while(*p++ == *line)
-	param+= 1;
+      while(*p == *line) {
+        p++;
+        param+= 1;
+      }
       if (*p)
 	goto badnumber;
     } else {
@@ -3370,7 +3403,13 @@ int mserv_setmixer(t_client *cl, int what, const char *line)
 	mserv_response(cl, "IOCTLWR", NULL);
 	return -1;
       }
-      newval = newval & 0xFF;
+      if (ioctl(mixer_fd, MIXER_READ(what), &newval) == -1) {
+        close(mixer_fd);
+        perror("iotcl read");
+        mserv_response(cl, "IOCTLRD", 0);
+        return -1;
+      }
+      newval = newval & 0xff;
       if (type == 0 || newval != curval)
 	break;
       param++;
